@@ -67,9 +67,9 @@ class FeaturesAccessor(BaseAccessor):
 
 
 	def _binary_encode(self, column):
-		'''
+		"""
 		Binary-encode a specific column.
-		'''
+		"""
 		assert self.is_categorical(column), 'The column should be categorical'
 		x = self._obj[column].values
 		x_ = np.array([str(_) for _ in x]).astype(str)
@@ -90,7 +90,7 @@ class FeaturesAccessor(BaseAccessor):
 		return res
 
 
-	def entity_features(self, entity, exclude=[]):
+	def entity_features(self, entity, exclude=[], entity_name='*'):
 		"""
 		Group rows corresponding to the same entity and apply aggregation functions.
 
@@ -113,52 +113,47 @@ class FeaturesAccessor(BaseAccessor):
 			The dataframe of features.
 		"""
 		assert entity in self._obj.columns, 'The entity %s should be valid column' % entity
-		entity_grp = self._obj.groupby(entity)
 
 		cat_columns = [col for col in self._obj.columns if self.is_categorical(col) and col != entity and col not in exclude]
 		ord_columns = [col for col in self._obj.columns if not self.is_categorical(col) and col != entity and col not in exclude]
 		columns = cat_columns + ord_columns
 
 		dfs = []
-
+		agg = {}
 		# Aggregation of categorical variables
 		if cat_columns:
 			# Most frequent value and it's frequency of occurence
-			cat_agg = {'MODE(%s)' % col: (col, mode) for col in cat_columns}
-			cat_agg.update({'MODEFREQ(%s)' % col: (col, modefreq) for col in cat_columns})
+			agg.update({'MODE(%s)' % col: (col, mode) for col in cat_columns})
+			agg.update({'MODEFREQ(%s)' % col: (col, modefreq) for col in cat_columns})
 
 			# Second most frequent value and it's frequency of occurence
-			cat_agg.update({'NEXTMODE(%s)' % col: (col, nextmode) for col in cat_columns})
-			cat_agg.update({'NEXTMODEFREQ(%s)' % col: (col, nextmodefreq) for col in cat_columns})
+			agg.update({'NEXTMODE(%s)' % col: (col, nextmode) for col in cat_columns})
+			agg.update({'NEXTMODEFREQ(%s)' % col: (col, nextmodefreq) for col in cat_columns})
 
 			# Least frequent value and it's frequency of occurence
-			cat_agg.update({'LASTMODE(%s)' % col: (col, lastmode) for col in cat_columns})
-			cat_agg.update({'LASTMODEFREQ(%s)' % col: (col, lastmodefreq) for col in cat_columns})
+			agg.update({'LASTMODE(%s)' % col: (col, lastmode) for col in cat_columns})
+			agg.update({'LASTMODEFREQ(%s)' % col: (col, lastmodefreq) for col in cat_columns})
 
-			cat_df = entity_grp.agg(**cat_agg)
-			dfs += [cat_df]
 
 		# Aggregation of ordinal variables
 		if ord_columns:
-			ord_agg = {'MEAN(%s)' % col: (col, nanmean) for col in ord_columns}
-			ord_agg.update({'STD(%s)' % col: (col, nanstd) for col in ord_columns})
-			ord_agg.update({'MEDIAN(%s)' % col: (col, nanmedian) for col in ord_columns})
-			ord_agg.update({'SKEW(%s)' % col: (col, nanskew) for col in ord_columns})
-			ord_agg.update({'KURT(%s)' % col: (col, nankurtosis) for col in ord_columns})
-			ord_agg.update({'Q25(%s)' % col: (col, q25) for col in ord_columns})
-			ord_agg.update({'Q75(%s)' % col: (col, q75) for col in ord_columns})
-			ord_agg.update({'MIN(%s)' % col: (col, nanmin) for col in ord_columns})
-			ord_agg.update({'MAX(%s)' % col: (col, nanmax) for col in ord_columns})
-			ord_agg.update({'MAX-MIN(%s)' % col: (col, nanmaxmmin) for col in ord_columns})
-			ord_df = entity_grp.agg(**ord_agg)
-			dfs += [ord_df]
+			agg.update({'MEAN(%s)' % col: (col, nanmean) for col in ord_columns})
+			agg.update({'STD(%s)' % col: (col, nanstd) for col in ord_columns})
+			agg.update({'MEDIAN(%s)' % col: (col, nanmedian) for col in ord_columns})
+			agg.update({'SKEW(%s)' % col: (col, nanskew) for col in ord_columns})
+			agg.update({'KURT(%s)' % col: (col, nankurtosis) for col in ord_columns})
+			agg.update({'Q25(%s)' % col: (col, q25) for col in ord_columns})
+			agg.update({'Q75(%s)' % col: (col, q75) for col in ord_columns})
+			agg.update({'MIN(%s)' % col: (col, nanmin) for col in ord_columns})
+			agg.update({'MAX(%s)' % col: (col, nanmax) for col in ord_columns})
+			agg.update({'MAX(%s)-MIN(%s)' % (col, col): (col, nanmaxmmin) for col in ord_columns})
 		
 		# Number of rows per entity
-		count_agg = {'COUNT(*)': (columns[0], 'count')}
-		count_df = entity_grp.agg(**count_agg)
-		dfs += [count_df]
+		agg.update({'COUNT(%s)' % entity_name: (columns[0], 'count')})
 
-		df = pd.concat(dfs, axis=1)
+		# Features
+		entity_grp = self._obj.groupby(entity)
+		df = entity_grp.agg(**agg)
 
 		return df
 
@@ -196,10 +191,10 @@ class FeaturesAccessor(BaseAccessor):
 		df = self._obj.copy()
 		if ord_columns:
 			for col in ord_columns:
-				df['| %s - MEAN(%s)|' % (col, col)] = np.abs(df[col]-means.loc[col])
-				df['| %s - MEDIAN(%s)|' % (col, col)] = np.abs(df[col]-quantiles.loc[0.5][col])
-				df['| %s - Q25(%s)|' % (col, col)] = np.abs(df[col]-quantiles.loc[0.25][col])
-				df['| %s - Q75(%s)|' % (col, col)] = np.abs(df[col]-quantiles.loc[0.75][col])
+				df['|%s - MEAN(%s)|' % (col, col)] = np.abs(df[col]-means.loc[col])
+				df['|%s - MEDIAN(%s)|' % (col, col)] = np.abs(df[col]-quantiles.loc[0.5][col])
+				df['|%s - Q25(%s)|' % (col, col)] = np.abs(df[col]-quantiles.loc[0.25][col])
+				df['|%s - Q75(%s)|' % (col, col)] = np.abs(df[col]-quantiles.loc[0.75][col])
 
 		if return_baselines:
 			return df, means, quantiles
@@ -242,7 +237,7 @@ class FeaturesAccessor(BaseAccessor):
 			dfs += [lagged_min_df.copy()]
 			lagged_max_df = rol_grp.aggregate(nanmax).rename(columns={col: 'MAX(%s, %d)' % (col, lag) for col in ord_columns})
 			dfs += [lagged_max_df.copy()]
-			lagged_maxmmin_df = rol_grp.aggregate(nanmaxmmin).rename(columns={col: 'MAX-MIN(%s, %d)' % (col, lag) for col in ord_columns})
+			lagged_maxmmin_df = rol_grp.aggregate(nanmaxmmin).rename(columns={col: 'MAX(%s, %d)-MIN(%s, %d)' % (col, lag, col, lag) for col in ord_columns})
 			dfs += [lagged_maxmmin_df.copy()]
 
 		df = pd.concat(dfs, axis=1)
@@ -251,7 +246,7 @@ class FeaturesAccessor(BaseAccessor):
 
 
 	def generate_features(self, entity=None, encoding_method='one_hot', index=None, max_lag=None, exclude=[], \
-			means=None, quantiles=None, return_baselines=False):
+			means=None, quantiles=None, return_baselines=False, entity_name='*'):
 		"""
 		Generate a wide range of candidate features to search from.
 
@@ -286,7 +281,7 @@ class FeaturesAccessor(BaseAccessor):
 		accessor = self
 		if entity:
 			# Entity features
-			df = accessor.entity_features(entity)
+			df = accessor.entity_features(entity, entity_name=entity_name)
 			accessor = FeaturesAccessor(df)
 
 		# Deviation features
